import {Component, OnDestroy, OnInit} from '@angular/core';
import {OmpClass} from 'src/app/dao/omp-class';
import {StudentInfoService} from 'src/app/services/student-info.service';
import {switchMap, takeWhile} from 'rxjs/operators';
import {ActivatedRoute, ParamMap} from '@angular/router';
import {MinutePaper} from 'src/app/dao/minute-paper';

@Component({
  selector: 'app-student-dashboard',
  templateUrl: './student-dashboard.component.html',
  styleUrls: ['./student-dashboard.component.css']
})
export class StudentDashboardComponent implements OnInit, OnDestroy {

  private isAlive = true;
  public hasClassList = false;
  public hasPaperList = false;
  public classList: OmpClass[];
  public openPaperList: MinutePaper[];
  public closedPaperList: MinutePaper[];
  public classPaperList: MinutePaper[];
  public studentId: number;

  constructor(private studentInfoService: StudentInfoService,
              private route: ActivatedRoute) {
  }

  ngOnInit(): void {
    this.studentId = this.route.paramMap['id'];
    this.classList = [];
    this.openPaperList = [];
    this.closedPaperList = [];
    this.classPaperList = [];
    this.getClassList();
    this.getOpenPapers();
  }

  ngOnDestroy(): void {
    this.isAlive = false;
  }

  private onDashboard(): Boolean {
    return !this.hasClassList && !this.hasPaperList;
  }

  private onClassList(): Boolean {
    return this.hasClassList;
  }

  private onPaperPage(): Boolean {
    return this.hasPaperList;
  }

  private getClassList() {
    this.route.paramMap.pipe(switchMap((params: ParamMap) => this.studentInfoService.getUserClasses(params.get('id'))))
      .subscribe((data: OmpClass[]) => {data.forEach(c => this.classList.push(c)); })
      .add(takeWhile(() => this.isAlive));
  }

  private getOpenPapers() {
    this.route.paramMap.pipe(switchMap((params: ParamMap) => this.studentInfoService.getOpenPapers(params.get('id'))))
      .subscribe((data: MinutePaper[]) => {data.forEach(c => this.openPaperList.push(c)); })
      .add(takeWhile(() => this.isAlive));
  }

  private getSubmittedPapers(classCode: String) {
    this.route.paramMap.pipe(switchMap((params: ParamMap) => this.studentInfoService.getSubmittedPapers(params.get('id'), classCode)))
      .subscribe((data: MinutePaper[]) => {data.forEach(c => this.closedPaperList.push(c)); })
      .add(takeWhile(() => this.isAlive));
  }

  private getOpenClassPapers(classCode: String) {
    this.openPaperList.forEach(e => {
      if (e.classCode === classCode) {
        this.classPaperList.push(e);
      }
    });
  }

  private buildClassPage(classCode: String) {
    this.hasClassList = !this.hasClassList;
    this.getOpenClassPapers(classCode);
    this.getSubmittedPapers(classCode);
  }

  private buildPaperPage(): void {
    this.hasPaperList = !this.hasPaperList;
  }
}
